# Installasjonsguide for Kotlin
Vi har utviklet dette biblioteket i Kotlin og vil anbefale at du som bruker også bruker Kotlin i ditt prosjekt. 
Kotlin kjører på JVM og kan derfor brukes sømløst i Java prosjekter. 
Det er også mulig å bruke bilbioteket med Java, men det vil antakelig kreve noe mer arbeid.

## Hvorfor Kotlin?
1. **Kompatibilitet med Java**: Kotlin er fullstendig kompatibelt med Java, og du kan derfor bruke Kotlin i Java prosjekter uten å måtte konvertere hele prosjektet til Kotlin.
2. **Tryggere språk**: Kotlin har færre muligheter for nullpointer exceptions enn Java, da det har innebygget støtte for nullsikkerhet.
3. **Lesbarhet**: Kotlin har en enklere og mer konsis syntax enn Java, som gjør koden mer lesbar.
4. **Moderne egenskaper**: Kotlin har moderne egenskaper som immutabilitet, funksjonell programmering, extension functions og data classes.
5. **Vegvesenets retning**: Mange tjenester i Statens Vegvesen bruker Kotlin eller har begynt en gradvis overgang.  

Hvis du likevel vil bruke Java så står du fritt til det, men vårt fokus har vært på utvikleropplevelsen i Kotlin. 
Enkelte ting, som extension functions og default arguments, vil bli mindre intuitive og kan kreve mer kode i Java. 


## Slik legger du Kotlin til i ditt prosjekt
### Maven
For å legge til Kotlin i ditt prosjekt, legg til følgende i din `pom.xml` fil:
```xml
<properties>
    <kotlin.version>2.1.0</kotlin.version> <!-- Replace with the latest version --> 
</properties>

<dependensies>
    <dependency>
        <groupId>org.jetbrains.kotlin</groupId>
        <artifactId>kotlin-stdlib</artifactId>
        <version>${kotlin.version}</version>
    </dependency>
</dependensies>

<build>
    <plugins>
        <plugin>
            <groupId>org.jetbrains.kotlin</groupId>
            <artifactId>kotlin-maven-plugin</artifactId>
            <version>${kotlin.version}</version>
            <executions>
                <execution>
                    <id>compile</id>
                    <goals>
                        <goal>compile</goal>
                    </goals>
                    <configuration>
                        <sourceDirs>
                            <sourceDir>${project.basedir}/src/main/kotlin</sourceDir>
                            <sourceDir>${project.basedir}/src/main/java</sourceDir>
                        </sourceDirs>
                    </configuration>
                </execution>
                <execution>
                    <id>test-compile</id>
                    <goals>
                        <goal>test-compile</goal>
                    </goals>
                    <configuration>
                        <sourceDirs>
                            <sourceDir>${project.basedir}/src/test/kotlin</sourceDir>
                            <sourceDir>${project.basedir}/src/test/java</sourceDir>
                        </sourceDirs>
                    </configuration>
                </execution>
            </executions>
        </plugin>
    </plugins>
</build>
``` 
For mer informasjon om Kotlin i Maven, se Kotlins [dokumentasjon](https://kotlinlang.org/docs/maven.html).

### Gradle - Groovy DSL
For å legge til Kotlin i ditt prosjekt, legg til følgende i din `build.gradle` fil:
```groovy
plugins {
    id 'org.jetbrains.kotlin.jvm' version '2.1.0' // Replace with the latest version
}

repositories {
    mavenCentral()
}

dependencies {
    implementation "org.jetbrains.kotlin:kotlin-stdlib"
}

kotlin {
    sourceSets {
        main {
            kotlin.srcDirs += 'src/main/kotlin'
            java.srcDirs += 'src/main/java'
        }
        test {
            kotlin.srcDirs += 'src/test/kotlin'
            java.srcDirs += 'src/test/java'
        }
    }
}

```

### Gradle - Kotlin DSL
For å legge til Kotlin i ditt prosjekt, legg til følgende i din `build.gradle.kts` fil:
```kotlin
plugins {
    kotlin("jvm") version "2.1.0" // Replace with the latest version
}

repositories {
    mavenCentral()
}

dependencies {
    implementation("org.jetbrains.kotlin:kotlin-stdlib")
}

kotlin {
    sourceSets {
        getByName("main") {
            kotlin.srcDirs("src/main/kotlin")
            java.srcDirs("src/main/java")
        }
        getByName("test") {
            kotlin.srcDirs("src/test/kotlin")
            java.srcDirs("src/test/java")
        }
    }
}
```
