# Meldingsformidler - Client

Klientbibliotek for sending av meldinger gjennom meldingsformidler.

# Brukerstøtte

Skulle det være noe du ikke finner svar på her, kontakt oss gjerne

- I Slack-kanalen [#meldingsformidling-brukerforum](https://vegvesen.slack.com/archives/C06HEUPNJ0N)
- Gjennom [Forvalter av tjenesten](https://intranett.vegvesen.no/fag/andre-fagomrader/ikt-faglig/ifi/ikt-losninger/Detail/?systemId=OI-7254E00F5D2E4F93BADC4F4CC9FE25D4)

# Bruk av Meldingsformidler
Meldingsformidler brukes til å sende post til personer og organisasjoner, gjennom Digital postkasse, Altinn, Brev eller E-post. 
Etter du har sendt en melding til Meldingsformidler kan du spørre om status for å se hvilken kanal meldingen går gjennom, om alt gikk bra eller om noe galt har skjedd på veien. 

`MeldingsformidlerClient` tilbyr et enkelt grensesnitt for å kommunisere med Meldingsformidler og vi anbefaler alle våre brukere å bruke denne. 
Hvis du likevel vil koble deg på direkte så ligger klassene åpne i klientbiblioteket, det skal være intuitivt hvordan disse henger sammen. 
Swagger og API-contract kommer senere. 

## API-endringer ved overgang til Rest
Vi har lenge ønsket å gå bort fra Soap. Nå bytter vi til Rest for å gjøre det enklere å bruke Meldingsformidler. Fra **1. juni 2025** vil Soap ikke lenger være tilgjengelig. 
Vi endrer også noen egenskaper ved API-et for Altinn, Digital post og Brev:

- **Fjerner mulighet for å velge kanal**. Vi ønsker at meldinger sendes til den kanalen mottaker foretrekker.
- **Bevis har ikke lengre et eget endepunkt**. Bevis er nå et valgfritt felt i meldingens innhold og sendes til samme endepunkt som andre meldinger.
- **Nytt endepunkt for å sjekke kanal**. Dette er for avsendere med spesielle behov for å vite hvilken kanal meldingen går gjennom på forhånd.


# Integrasjon

## Bestille tilgang til Meldingsformidler

For å kunne sende meldinger gjennom Meldingsformidler så må man ha en _tjenestebruker_. 

**Detaljer for bestilling [finner du her](Tilgang.md).**

## Swagger/OpenApi docs

- [Utv](https://meldingsmottak.utv.atlas.vegvesen.no/swagger-ui/index.html)
- [Atm](https://meldingsmottak.test.atlas.vegvesen.no/swagger-ui/index.html)
- [Produksjon](https://meldingsmottak.atlas.vegvesen.no/swagger-ui/index.html)

## Installere klienten som en avhengighet

Legg til `meldingsformidler-client` som avhengighet i ditt prosjekt. Siste versjonsnummer finner du i [Artifactory (Artrepo)](https://artrepo.vegvesen.no/ui/native/libs-release-local/no/vegvesen/meldingsformidler/meldingsformidler-client/). Under ser du hvordan det skal legges inn som avhengighet for henholdsvis Maven og Gradle.

### Maven

```xml

<dependency>
    <groupId>no.vegvesen.meldingsformidler</groupId>
    <artifactId>meldingsformidler-client</artifactId>
    <version>SETT VERSJONSNUMMER</version>
</dependency>
```

### Gradle

```gradle
implementation 'no.vegvesen.meldingsformidler:meldingsformidler-client:SETT VERSJONSNUMMER' 
```

## Konfigurere klienten

Vi anbefaler at du skriver Kotlin når du bruker klienten. Biblioteket er skrevet i Kotlin og vil gi deg en bedre utvikleropplevelse enn Java. Dersom du ikke har Kotlin i ditt prosjekt, så kan du følge vår [Installasjonsguide for Kotlin](Kotlin.md).
Her forklarer vi også hvorfor vi anbefaler Kotlin. 

For å kunne sende meldinger gjennom Meldingsformidler så må man være autentisert. For de som ikke setter autentiseringsheadere selv så må autentiseringen gjøres med en `OpenAmClient`.

```kt
val openAmUrl = "URL til OpenAM i miljøet du skal sende meldinger til"

val openIdClientId = " Bruk din tjeneste sitt Oauth-brukernavn"
val openIdClientSecret = "Bruk din tjeneste sitt Oauth-passord"

val tjenestebruker = "Tjenestebuker for å hente token, f.eks. TjeDinTjeneste_Melding. Denne må lages av oss."
val tjenestebrukerPassword = "Passord for tjenestebrukeren"

val openAmClient = OpenAmClient(
    Oauth2Client.create(
        openAmUrl = openAmUrl,
        openAmUsername = tjenestebruker,
        openAmPassord = tjenestebrukerPassword,
        oidcClientId = openIdClientId,
        oidcClientSecret = openIdClientSecret
    )
)
```

_example:openAmClient_

Dette er url til OpenAm i de forskjellige miljøene:

- [STM](https://www.utv.vegvesen.no/openam/oauth2/realms/root/realms/WSClients_Int/access_token)
- [ATM](https://www.test.vegvesen.no/openam/oauth2/realms/root/realms/WSClients_Int/access_token)
- [PROD](https://www.vegvesen.no/openam/oauth2/realms/root/realms/WSClients_Int/access_token)

OpenAm-klienten kan sendes inn som parameter til `MeldingsformidlerClient`:

```kt

val meldingsformidlerClient = MeldingsformidlerClient(MeldingsformidlerEnvironment.Systemtest, openAmClient)

```
_example:clientSetup_

# Opprette en melding

En melding består i hovedsak av tre deler:
- Innhold
- Mottaker
- Konfigurasjon for tjenestene meldingen kan sendes til

## Innhold

Du kan enten sende en dokumentmelding eller en e-postmelding.

### Dokumentmeldinger
Avhengig av hvem som er mottaker vil meldingen bli sendt til

- Altinn
- Offentlig post - gjennom Digipost, E-boks eller brev i posten.

Innholdet i meldingen består av en tittel og dokumenter. Det er påkrevd å ha et hoveddokument, og du har mulighet til å legge ved en liste med vedlegg. Dokumentene må være i Pdf-format.

```kt
val dokumentInnhold: DokumentInnhold = DokumentInnhold(
    tittel = "Vedtak om veivalg for E6 Kvænangsfjellet",
    hoveddokument = Dokument(tittel = "Primært beslutningsdokument", bytes = byteArrayOf(1, 2, 3)),
    vedlegg = listOf(
        Dokument(tittel = "Referat", bytes = byteArrayOf(4, 5, 6))
    )
)
```

_example:innhold_

### E-postmeldinger
Innholdet i en e-postmelding består av et emne og innhold.

```kt
val epostInnhold: EpostInnhold = EpostInnhold(
    emne = "En epost fra Vegvesenet",
    innhold = "Melding til deg fra Vegvesenet"
)
```
_example:epostInnhold_

## Mottaker

### Dokumentmeldinger

Mottakeren av meldingen er en person eller organisasjon og identifiseres med fødselsnummer eller organisasjonsnummer.

```kt
val organisasjonMottaker: Organisasjon = Organisasjon("12345678900")
val personMottaker: Person = Person("12345678900")
```

_example:mottaker_

### E-postmeldinger

Mottakeren av meldingen er en e-postmottaker med en e-postadresse.

```kt
val epostMottaker: EpostMottaker = EpostMottaker("hei@example.com")
```

_example:epostMottaker_

## Konfigurasjon
Meldingsformidler trenger litt metadata for å kunne sende meldingen. 

### Dokumentmeldinger

Avhengig av hvem som er mottaker vil meldingen bli sendt til

- Altinn
- Offentlig post - gjennom Digipost, E-boks eller brev i posten.

Derfor må du ha en konfigurasjon for Altinn og OffentligPost.

Meldingsformidler sender meldingen til den tjenesten som mottaker foretrekker.
Det er satt default-verdier for både Altinn og OffentligPost, så det er ikke nødvendig å sette disse om du ikke har spesielle behov. Det er likevel viktig å ta stilling til alle feltene.

```kt
val altinnConfiguration = AltinnConfiguration()
val offentligPostConfiguration = OffentligPostConfiguration()

val dokumentConfiguration: DokumentConfiguration = DokumentConfiguration(
    sprak = Sprak.Bokmal,
    iktLosningId = IktLosningId("<din-iktløsning-id>"),
    altinnConfiguration = altinnConfiguration,
    offentligPostConfiguration = offentligPostConfiguration
)
```

_example:configuration_

### E-postmeldinger

For e-post bruker du en `EpostConfiguration`.

```kt
val epostConfiguration = EpostConfiguration(
    iktLosningId = IktLosningId("<din-iktløsning-id>"),
)
```
_example:epostConfiguration_


# Bygge opp en melding

Når du har satt opp innhold, mottaker og konfigurasjon så kan du bygge opp en melding.

```kt
val dokumentMelding: DokumentMelding = DokumentMelding(dokumentInnhold, personMottaker, dokumentConfiguration)
val epostMelding: EpostMelding = EpostMelding(epostInnhold, epostMottaker, epostConfiguration)
```
_example:buildMelding_

# Sende melding
Sammen med meldingen må du sende med en `IdempotencyKey`. Denne brukes for å unngå at samme melding blir sendt flere ganger. Derfor skal ikke dette være en tilfeldig verdi, men en verdi som er unik for hver melding i ditt system. 

Ved en vellykket sending får du en `SentMelding` tilbake. Denne inneholder en MeldingUuid som du videre skal bruke til å hente status på meldingen.

```kt
fun send(melding: Melding, idempotencyKey: IdempotencyKey) {
    val meldingResult: MeldingsformidlerResult<SentMelding> = meldingsformidlerClient.send(melding, idempotencyKey)
    return meldingResult.fold(
        onSuccess = { sentMelding ->
            // Meldingen er sendt og du har fått en SentMelding som har en MeldingUuid som du kan bruke til å hente status på meldingen.
            sentMelding
        },
        onFailure = { meldingsformidlerError ->
            // Noe gikk galt i sendingen, og du har fått enten en ClientError eller en UnexpectedError som må håndteres.
            meldingsformidlerError.fold(
                onClientError = ::handleClientError,
                onUnexpectedError = ::handleUnexpectedError
            )
        }
    )
}
```
_example:sendMelding_

# Hent status på melding

Når meldingen er sendt bruker du `meldingUuid` for å hente statusen på sendingen. Du får tilbake en `StatusMelding` som inneholder informasjon om kanal og siste hendelse.

```kt
fun updatedStatus(meldingUuid: MeldingUuid) =
    meldingsformidlerClient
        .status(meldingUuid)
        .fold(
            onSuccess = { statusMelding ->
                // Statusen er hentet og du kan prosessere den videre.
                processStatus(statusMelding.lastHendelse.status)
            },
            onFailure = { meldingsformidlerError ->
                // Noe gikk galt i henting av status, og du har fått enten en ClientError eller en UnexpectedError som må håndteres.
                meldingsformidlerError.fold(
                    onClientError = ::handleClientError,
                    onUnexpectedError = ::handleUnexpectedError
                )
            }
        )
```

_example:status_

## Feilhåndtering

Dersom noe går galt i et API-kall får du tilbake en `MeldingsformidlerFailure`. Den inneholder enten en `ClientError` eller `UnexpectedError`.
`ClientError` er en forventet feilsituasjon som inneholder en [ProblemDetail](https://docs.spring.io/spring-framework/docs/current/javadoc-api/org/springframework/http/ProblemDetail.html) av hva som gikk galt, mens `UnexpectedError` er en uforventet feil som ikke er håndtert av Meldingsformidler,
og inneholder en `error` av typen `Throwable`.

`MeldingsformidlerFailure` har en `fold`-funksjon som tar to lambdafunksjoner, en for å håndtere `ClientError` og en for å håndtere `UnexpectedError`.
De kan se slik ut:

```kt
fun handleClientError(clientError: ClientError) {
    println(clientError.problemDetail)
    // Noe gikk galt under sending av forespørselen, men det er håndtert av meldingsformidler
    // og du får en beskrivelse av hva som gikk galt.
}

fun handleUnexpectedError(unexpectedError: UnexpectedError) {
    println(unexpectedError.throwable)
    // En uforventet feil har oppstått. Dette er en feil som ikke er håndtert av meldingsformidler,
    // og du vil få en Throwable som du kan bruke til å finne ut hva som gikk galt.
}
```
_example:handleErrors_


## Behandle status

Statusen på en melding sier noe om hva som har skjedd med meldingen. I tillegg har den et felt, `NextSenderAction`, som sier hva du som avsender må gjøre videre. Under er et oppsett for hvordan du kan behandle statusen på en melding.

```kt
fun processStatus(status: Status) {
    when (status) {
        Status.Processing -> {
            // Meldingen er under behandling og avsender trenger ikke gjøre noe.
            // Alt er som det skal. Hent ny status om en stund.
        }

        Status.TemporaryErrorRetrying -> {
            // Det har oppstått en midlertidig feil som Meldingsformidler vil prøve å rette opp selv.
            // Du som avsender trenger ikke gjøre noe foreløpig.
        }

        Status.ProcessedAndSent -> {
            // Meldingen er sendt til mottaker og meldingen er ferdig behandlet.
            // Prosessen er ferdig og du trenger ikke gjøre noe mer.
        }

        Status.UnableToSend -> {
            if (status.nextSenderAction == NextSenderAction.HandleManually) {
                // Noe uventet har skjedd som ikke kan håndteres automatisk. Ta kontakt med oss i #meldingsformidling-brukerforum
            } else if (status.nextSenderAction == NextSenderAction.PollForNextStatus) {
                // Noe uventet har skjedd og Meldingsformidler vil prøve å sende meldingen igjen.
                // Hent ny status om en stund.
            }
        }
    }
}
```

_example:processStatus_

## Samlet eksempel
```kt
import no.vegvesen.meldingsformidler.MeldingsformidlerClient
import no.vegvesen.meldingsformidler.MeldingsformidlerEnvironment
import no.vegvesen.meldingsformidler.common.domain.Dokument
import no.vegvesen.meldingsformidler.common.domain.IktLosningId
import no.vegvesen.meldingsformidler.common.domain.MeldingUuid
import no.vegvesen.meldingsformidler.common.domain.Sprak
import no.vegvesen.meldingsformidler.common.openamclient.Oauth2Client
import no.vegvesen.meldingsformidler.common.openamclient.OpenAmClient
import no.vegvesen.meldingsformidler.send.AltinnConfiguration
import no.vegvesen.meldingsformidler.send.ClientError
import no.vegvesen.meldingsformidler.send.DokumentConfiguration
import no.vegvesen.meldingsformidler.send.DokumentInnhold
import no.vegvesen.meldingsformidler.send.DokumentMelding
import no.vegvesen.meldingsformidler.send.EpostConfiguration
import no.vegvesen.meldingsformidler.send.EpostInnhold
import no.vegvesen.meldingsformidler.send.EpostMelding
import no.vegvesen.meldingsformidler.send.EpostMottaker
import no.vegvesen.meldingsformidler.send.IdempotencyKey
import no.vegvesen.meldingsformidler.send.Melding
import no.vegvesen.meldingsformidler.send.MeldingsformidlerResult
import no.vegvesen.meldingsformidler.send.OffentligPostConfiguration
import no.vegvesen.meldingsformidler.send.Organisasjon
import no.vegvesen.meldingsformidler.send.Person
import no.vegvesen.meldingsformidler.send.SentMelding
import no.vegvesen.meldingsformidler.send.UnexpectedError
import no.vegvesen.meldingsformidler.status.NextSenderAction
import no.vegvesen.meldingsformidler.status.Status

val openAmUrl = "URL til OpenAM i miljøet du skal sende meldinger til"

val openIdClientId = " Bruk din tjeneste sitt Oauth-brukernavn"
val openIdClientSecret = "Bruk din tjeneste sitt Oauth-passord"

val tjenestebruker = "Tjenestebuker for å hente token, f.eks. TjeDinTjeneste_Melding. Denne må lages av oss."
val tjenestebrukerPassword = "Passord for tjenestebrukeren"

val openAmClient = OpenAmClient(
    Oauth2Client.create(
        openAmUrl = openAmUrl,
        openAmUsername = tjenestebruker,
        openAmPassord = tjenestebrukerPassword,
        oidcClientId = openIdClientId,
        oidcClientSecret = openIdClientSecret
    )
)


val meldingsformidlerClient = MeldingsformidlerClient(MeldingsformidlerEnvironment.Systemtest, openAmClient)


val dokumentInnhold: DokumentInnhold = DokumentInnhold(
    tittel = "Vedtak om veivalg for E6 Kvænangsfjellet",
    hoveddokument = Dokument(tittel = "Primært beslutningsdokument", bytes = byteArrayOf(1, 2, 3)),
    vedlegg = listOf(
        Dokument(tittel = "Referat", bytes = byteArrayOf(4, 5, 6))
    )
)

val epostInnhold: EpostInnhold = EpostInnhold(
    emne = "En epost fra Vegvesenet",
    innhold = "Melding til deg fra Vegvesenet"
)

val organisasjonMottaker: Organisasjon = Organisasjon("12345678900")
val personMottaker: Person = Person("12345678900")

val epostMottaker: EpostMottaker = EpostMottaker("hei@example.com")

val altinnConfiguration = AltinnConfiguration()
val offentligPostConfiguration = OffentligPostConfiguration()

val dokumentConfiguration: DokumentConfiguration = DokumentConfiguration(
    sprak = Sprak.Bokmal,
    iktLosningId = IktLosningId("<din-iktløsning-id>"),
    altinnConfiguration = altinnConfiguration,
    offentligPostConfiguration = offentligPostConfiguration
)

val epostConfiguration = EpostConfiguration(
    iktLosningId = IktLosningId("<din-iktløsning-id>"),
)

val dokumentMelding: DokumentMelding = DokumentMelding(dokumentInnhold, personMottaker, dokumentConfiguration)
val epostMelding: EpostMelding = EpostMelding(epostInnhold, epostMottaker, epostConfiguration)

fun send(melding: Melding, idempotencyKey: IdempotencyKey) {
    val meldingResult: MeldingsformidlerResult<SentMelding> = meldingsformidlerClient.send(melding, idempotencyKey)
    return meldingResult.fold(
        onSuccess = { sentMelding ->
            // Meldingen er sendt og du har fått en SentMelding som har en MeldingUuid som du kan bruke til å hente status på meldingen.
            sentMelding
        },
        onFailure = { meldingsformidlerError ->
            // Noe gikk galt i sendingen, og du har fått enten en ClientError eller en UnexpectedError som må håndteres.
            meldingsformidlerError.fold(
                onClientError = ::handleClientError,
                onUnexpectedError = ::handleUnexpectedError
            )
        }
    )
}

fun updatedStatus(meldingUuid: MeldingUuid) =
    meldingsformidlerClient
        .status(meldingUuid)
        .fold(
            onSuccess = { statusMelding ->
                // Statusen er hentet og du kan prosessere den videre.
                processStatus(statusMelding.lastHendelse.status)
            },
            onFailure = { meldingsformidlerError ->
                // Noe gikk galt i henting av status, og du har fått enten en ClientError eller en UnexpectedError som må håndteres.
                meldingsformidlerError.fold(
                    onClientError = ::handleClientError,
                    onUnexpectedError = ::handleUnexpectedError
                )
            }
        )

fun handleClientError(clientError: ClientError) {
    println(clientError.problemDetail)
    // Noe gikk galt under sending av forespørselen, men det er håndtert av meldingsformidler
    // og du får en beskrivelse av hva som gikk galt.
}

fun handleUnexpectedError(unexpectedError: UnexpectedError) {
    println(unexpectedError.throwable)
    // En uforventet feil har oppstått. Dette er en feil som ikke er håndtert av meldingsformidler,
    // og du vil få en Throwable som du kan bruke til å finne ut hva som gikk galt.
}

fun processStatus(status: Status) {
    when (status) {
        Status.Processing -> {
            // Meldingen er under behandling og avsender trenger ikke gjøre noe.
            // Alt er som det skal. Hent ny status om en stund.
        }

        Status.TemporaryErrorRetrying -> {
            // Det har oppstått en midlertidig feil som Meldingsformidler vil prøve å rette opp selv.
            // Du som avsender trenger ikke gjøre noe foreløpig.
        }

        Status.ProcessedAndSent -> {
            // Meldingen er sendt til mottaker og meldingen er ferdig behandlet.
            // Prosessen er ferdig og du trenger ikke gjøre noe mer.
        }

        Status.UnableToSend -> {
            if (status.nextSenderAction == NextSenderAction.HandleManually) {
                // Noe uventet har skjedd som ikke kan håndteres automatisk. Ta kontakt med oss i #meldingsformidling-brukerforum
            } else if (status.nextSenderAction == NextSenderAction.PollForNextStatus) {
                // Noe uventet har skjedd og Meldingsformidler vil prøve å sende meldingen igjen.
                // Hent ny status om en stund.
            }
        }
    }
}
```
_example:fullExample_


## Grafisk brukergrensesnitt - GUI

Meldingsformidler har også et grafisk brukergrensesnitt som kan brukes til å søke etter meldinger og sjekke status. Her må man først ha tilgang.

### Hvordan få tilgang

Du trenger `Meldingsformidler - Lese logg` i riktig miljø.

1. Gå til selvbetjeningsportalen
   for [STM/UTV](https://www.utv.vegvesen.no/idportalen/selvbetjening/), [ATM](https://www.test.vegvesen.no/idportalen/selvbetjening/)
   eller [PROD](https://www.vegvesen.no/idportalen/selvbetjening/).
2. Søk opp `Meldingsformidler - Lese logg` og be om tilgang.
    1. Den vil bli automatisk godkjent i STM/UTV og ATM, men må godkjennes av forvalter i PROD.
3. Logg inn i
   meldingsloggen [STM/UTV](https://meldingslogg.utv.atlas.vegvesen.no/meldingsformidler/meldingslogg/home), [ATM](https://meldingslogg.test.atlas.vegvesen.no/meldingsformidler/meldingslogg/home)
   eller [PROD](https://meldingslogg.atlas.vegvesen.no/meldingsformidler/meldingslogg/home).
          

## Hvordan se melding i postkassen
For å sjekke at meldingen har kommet frem til mottaker kan du sende meldingen til en syntetisk person eller organisasjon, og deretter logge inn i postkassen til mottakeren. Husk at dette er offentlige brukere, så ikke send sensitiv informasjon.  
                      
### Postkasse i Digipost
1. Send melding til fødselsnummer 16823549575 (KOGNITIV INNSJØ)
2. Logg inn i [Difitest Digipost](https://www.difitest.digipost.no/) med ID-porten og bruk 16823549575 som TestID
3. Se meldingen din i innboksen
4. Dersom du har lyst til å bruke egne testbrukere, så kan du opprette postkasse til tilfeldige syntetiske personer:
   1. Logg deg inn i [Kontakt- og reservasjonsregisteret sitt testmiljø](https://minprofil.test.kontaktregisteret.no/) med `TestId på høyt nivå`
   2. Trykk `Hent tilfeldig person` og `Autentiser`. Ta vare på fødselsnummeret.
   3. Opprett postkasse ved å velge `Din digitale postkasse` og så velge postkassen du vil opprette
   4. Det vil nå kunne ta litt tid før postkassen er aktiv (har erfart 2 dager)
   5. Logg inn i [Difitest Digipost](https://www.difitest.digipost.no/) eller [E-boks demo](https://demo2-www.e-boks.no/privat) avhengig av hvilken postkasse du valgte

### Postkasse i Altinn
1. Send melding til fødselsnummer 11887197158 (ULIK BOKKLUBB)
2. Logg inn i [Altinn testmiljø](https://info.tt02.altinn.no/no/) med `TestID på høyt nivå`
3. Se meldingen din i innboksen
4. Dersom du har lyst til å bruke egne testbrukere, så kan du finne en bruker gjennom [Tenor](https://www.skatteetaten.no/skjema/testdata/) som har `NEI` på reservasjon i Kontakt og reservasjonsregisteret                                           


Brukerene som vi har i [IntegrationTestData](src/test/java/no/vegvesen/meldingsformidler/integrationtests/IntegrationTestData.kt) brukes i våre integrasjonstester, derfor ønsker vi ikke at andre skal bruke disse.
