package no.vegvesen.meldingsformidler.send

import no.vegvesen.meldingsformidler.common.domain.ContentHasher
import no.vegvesen.mottak.rest.dto.SendDokumentMeldingRequestV2
import no.vegvesen.mottak.rest.dto.SendEpostConfigurationV2Dto
import no.vegvesen.mottak.rest.dto.SendEpostInnholdV2Dto
import no.vegvesen.mottak.rest.dto.SendEpostMottakerV2Dto
import no.vegvesen.mottak.rest.dto.SendEpostRequestV2


sealed interface Melding {
    val innhold: Innhold
    val mottaker: Mottaker
    val configuration: Configuration
}

data class DokumentMelding(
    override val innhold: DokumentInnhold,
    override val mottaker: DokumentMottaker,
    override val configuration: DokumentConfiguration,
) : Melding


data class EpostMelding(
    override val innhold: EpostInnhold,
    override val mottaker: EpostMottaker,
    override val configuration: EpostConfiguration,
) : Melding


internal fun DokumentMelding.toDto(): SendDokumentMeldingRequestV2 =
    SendDokumentMeldingRequestV2(
        mottaker = mottaker.toDto(),
        innhold = innhold.toDto(),
        configuration = configuration.toDto()
    )

internal fun EpostMelding.toDto(): SendEpostRequestV2 {
    return SendEpostRequestV2(
        mottaker = SendEpostMottakerV2Dto(
            epost = mottaker.epostAdresse.value
        ),
        innhold = SendEpostInnholdV2Dto(
            emne = innhold.emne,
            innhold = innhold.innhold
        ),
        configuration = SendEpostConfigurationV2Dto(
            iktLosning = configuration.iktLosningId.value
        )
    )

}

internal fun SendDokumentMeldingRequestV2.toHash(): String =
    ContentHasher.dokumentInnholdHash(
        innhold.tittel,
        innhold.hoveddokument.tittel,
        innhold.hoveddokument.bytes,
        mottaker.identifikator
    )

internal fun SendEpostRequestV2.toHash(): String =
    ContentHasher.epostInnholdHash(
        innhold.emne,
        innhold.innhold,
        mottaker.epost
    )