package no.vegvesen.meldingsformidler.send

import no.vegvesen.meldingsformidler.common.domain.AdresseLinjer
import no.vegvesen.meldingsformidler.common.domain.Epostadresse
import no.vegvesen.meldingsformidler.common.domain.Fodselsnummer
import no.vegvesen.meldingsformidler.common.domain.Organisasjonsnummer
import no.vegvesen.mottak.rest.dto.DokumentMeldingAdresseLinjerV2Dto
import no.vegvesen.mottak.rest.dto.DokumentMeldingMottakerV2Dto
import no.vegvesen.mottak.rest.dto.DokumentMeldingOverstyrtAdresseNorskV2Dto
import no.vegvesen.mottak.rest.dto.DokumentMeldingOverstyrtAdresseUtenlandskV2Dto
import no.vegvesen.mottak.rest.dto.DokumentMeldingOverstyrtAdresseV2Dto

sealed interface Mottaker {
    fun getId(): String
}

sealed interface DokumentMottaker : Mottaker

data class EpostMottaker(
    val epostAdresse: Epostadresse,
) : Mottaker {
    constructor(epostAdresse: String) : this(Epostadresse(epostAdresse))

    override fun getId(): String = epostAdresse.value
}

data class Person(
    val personIdentifikator: Fodselsnummer,
    val overstyrtAdresse: OverstyrtAdresse? = null,
) : DokumentMottaker {
    constructor(personIdentifikator: String) : this(Fodselsnummer(personIdentifikator))

    override fun getId(): String = personIdentifikator.value

}

data class Organisasjon(
    val organisasjonsnummer: Organisasjonsnummer,
) : DokumentMottaker {
    constructor(organisasjonsnummer: String) : this(Organisasjonsnummer(organisasjonsnummer))

    override fun getId(): String = organisasjonsnummer.value

}

sealed interface OverstyrtAdresse {
    val adresseLinjer: AdresseLinjer
    fun toDto(): DokumentMeldingOverstyrtAdresseV2Dto
}

data class OvertstyrtAdresseNorsk(
    val postnummer: String,
    val poststed: String,
    override val adresseLinjer: AdresseLinjer,
) : OverstyrtAdresse {
    override fun toDto(): DokumentMeldingOverstyrtAdresseV2Dto =
        DokumentMeldingOverstyrtAdresseV2Dto(
            overstyrtAdresseNorsk = DokumentMeldingOverstyrtAdresseNorskV2Dto(
                postnummer = postnummer,
                poststed = poststed,
                adresseLinjer = DokumentMeldingAdresseLinjerV2Dto(
                    linje1 = adresseLinjer.linje1,
                    linje2 = adresseLinjer.linje2,
                    linje3 = adresseLinjer.linje3,
                    linje4 = adresseLinjer.linje4
                ),
            )
        )
}

data class OvertstyrtAdresseUtenlandsk(
    val land: String,
    override val adresseLinjer: AdresseLinjer,
) : OverstyrtAdresse {
    override fun toDto(): DokumentMeldingOverstyrtAdresseV2Dto =
        DokumentMeldingOverstyrtAdresseV2Dto(
            overstyrtAdresseUtenlandsk = DokumentMeldingOverstyrtAdresseUtenlandskV2Dto(
                land = land,
                adresseLinjer = DokumentMeldingAdresseLinjerV2Dto(
                    linje1 = adresseLinjer.linje1,
                    linje2 = adresseLinjer.linje2,
                    linje3 = adresseLinjer.linje3,
                    linje4 = adresseLinjer.linje4
                ),
            )
        )
}

internal fun DokumentMottaker.toDto(): DokumentMeldingMottakerV2Dto = when (this) {
    is Person -> DokumentMeldingMottakerV2Dto(
        type = DokumentMeldingMottakerV2Dto.Type.Person,
        identifikator = personIdentifikator.value,
        overstyrtAdresse = overstyrtAdresse?.toDto()
    )

    is Organisasjon -> DokumentMeldingMottakerV2Dto(
        type = DokumentMeldingMottakerV2Dto.Type.Organisasjon,
        identifikator = organisasjonsnummer.value,
    )
}