package no.vegvesen.meldingsformidler.status

import no.vegvesen.meldingsformidler.common.domain.Kanal
import no.vegvesen.meldingsformidler.common.domain.MeldingUuid
import no.vegvesen.mottak.rest.dto.MeldingStatusHendelseV2Dto
import no.vegvesen.mottak.rest.dto.MeldingStatusResponseV2
import java.time.Instant

data class StatusMelding(
    val id: MeldingUuid,
    val lastHendelse: Hendelse
) {

    companion object {

        fun MeldingStatusResponseV2.toInternal(): StatusMelding =
            StatusMelding(
                id = MeldingUuid(meldingUuid),
                lastHendelse = Hendelse(
                    timestamp = hendelse.timestamp,
                    status = when (hendelse.status) {
                        MeldingStatusHendelseV2Dto.Status.ProcessedAndSent -> Status.ProcessedAndSent
                        MeldingStatusHendelseV2Dto.Status.Processing -> Status.Processing
                        MeldingStatusHendelseV2Dto.Status.TemporaryErrorRetrying -> Status.TemporaryErrorRetrying
                        MeldingStatusHendelseV2Dto.Status.UnableToSend -> Status.UnableToSend
                    },
                    kanal = when (hendelse.kanal) {
                        MeldingStatusHendelseV2Dto.Kanal.DigitalPost -> Kanal.DigitalPost
                        MeldingStatusHendelseV2Dto.Kanal.Epost -> Kanal.Epost
                        MeldingStatusHendelseV2Dto.Kanal.Brev -> Kanal.Brev
                        MeldingStatusHendelseV2Dto.Kanal.Altinn -> Kanal.Altinn
                    },
                )
            )
    }
}

data class Hendelse(
    val timestamp: Instant,
    val status: Status,
    val kanal: Kanal,
)

enum class Status(
    val nextSenderAction: NextSenderAction,
) {
    ProcessedAndSent(nextSenderAction = NextSenderAction.StopPollingForStatus),

    Processing(nextSenderAction = NextSenderAction.PollForNextStatus),
    TemporaryErrorRetrying(nextSenderAction = NextSenderAction.PollForNextStatus),

    UnableToSend(nextSenderAction = NextSenderAction.HandleManually);

}

enum class NextSenderAction {
    StopPollingForStatus,
    HandleManually,
    PollForNextStatus,
}
