import no.vegvesen.meldingsformidler.common.domain.AdresseLinjer
import no.vegvesen.meldingsformidler.common.domain.AdresseType
import no.vegvesen.meldingsformidler.common.domain.AltinnTjenestedefinisjon
import no.vegvesen.meldingsformidler.common.domain.ColorHex
import no.vegvesen.meldingsformidler.common.domain.Dokument
import no.vegvesen.meldingsformidler.common.domain.Fodselsnummer
import no.vegvesen.meldingsformidler.common.domain.Gyldighetsperiode
import no.vegvesen.meldingsformidler.common.domain.IktLosningId
import no.vegvesen.meldingsformidler.common.domain.IktLosningReferanse
import no.vegvesen.meldingsformidler.common.domain.MeldingUuid
import no.vegvesen.meldingsformidler.common.domain.Postadresse
import no.vegvesen.meldingsformidler.common.domain.PostadresseNorsk
import no.vegvesen.meldingsformidler.common.domain.Returhandtering
import no.vegvesen.meldingsformidler.common.domain.Sikkerhetsniva
import no.vegvesen.meldingsformidler.common.domain.Sprak
import no.vegvesen.meldingsformidler.common.domain.Tjenestekode
import no.vegvesen.meldingsformidler.common.domain.Tjenesteutgave
import no.vegvesen.meldingsformidler.send.AltinnConfiguration
import no.vegvesen.meldingsformidler.send.Bevis
import no.vegvesen.meldingsformidler.send.BevisAdresse
import no.vegvesen.meldingsformidler.send.BevisAttributt
import no.vegvesen.meldingsformidler.send.BevisHolder
import no.vegvesen.meldingsformidler.send.BevisInfo
import no.vegvesen.meldingsformidler.send.DokumentConfiguration
import no.vegvesen.meldingsformidler.send.DokumentInnhold
import no.vegvesen.meldingsformidler.send.DokumentMelding
import no.vegvesen.meldingsformidler.send.DokumentMottaker
import no.vegvesen.meldingsformidler.send.EpostConfiguration
import no.vegvesen.meldingsformidler.send.EpostInnhold
import no.vegvesen.meldingsformidler.send.EpostMelding
import no.vegvesen.meldingsformidler.send.EpostMottaker
import no.vegvesen.meldingsformidler.send.IdempotencyKey
import no.vegvesen.meldingsformidler.send.Melding
import no.vegvesen.meldingsformidler.send.Name
import no.vegvesen.meldingsformidler.send.OffentligPostConfiguration
import no.vegvesen.meldingsformidler.send.OverstyrtAdresse
import no.vegvesen.meldingsformidler.send.OvertstyrtAdresseNorsk
import no.vegvesen.meldingsformidler.send.OvertstyrtAdresseUtenlandsk
import no.vegvesen.meldingsformidler.send.Person
import java.time.Instant
import java.time.LocalDate
import java.util.UUID

object ArbitraryTestData {
    fun arbitraryDokumentMelding(
        dokumentInnhold: DokumentInnhold = arbitraryDokumentInnhold(),
        mottaker: DokumentMottaker = arbitraryPersonMottaker(),
        dokumentConfiguration: DokumentConfiguration = arbitraryDokumentConfiguration(),
    ): DokumentMelding {
        return DokumentMelding(
            innhold = dokumentInnhold,
            mottaker = mottaker,
            configuration = dokumentConfiguration
        )
    }

    fun arbitrayEpostMelding(
        dokumentInnhold: EpostInnhold = arbitraryEpostInnhold(),
        mottaker: EpostMottaker = arbitraryEpostMottaker(),
        configuration: EpostConfiguration = arbitraryEpostConfiguration()
    ): EpostMelding {
        return EpostMelding(
            innhold = dokumentInnhold,
            mottaker = mottaker,
            configuration = configuration
        )
    }

    fun arbitraryEpostConfiguration(
        iktLosningId: IktLosningId = arbitrayIktLosningId()
    ): EpostConfiguration = EpostConfiguration(
        iktLosningId = iktLosningId
    )

    fun arbitraryEpostMottaker(
        epostAdresse: EpostMottaker = EpostMottaker("hei@example.com")
    ): EpostMottaker {
        return epostAdresse
    }


    fun arbitraryEpostInnhold(
        emne: String = "Dette er et emne",
        innhold: String = "Dette er innholdet i eposten"
    ): EpostInnhold {
        return EpostInnhold(
            emne = emne,
            innhold = innhold,
        )
    }

    fun arbitraryBevisMelding(
        dokumentInnhold: DokumentInnhold = arbitraryDokumentInnholdWithBevis(),
        mottaker: DokumentMottaker = arbitraryPersonMottaker(),
        dokumentConfiguration: DokumentConfiguration = arbitraryDokumentConfiguration()
    ): Melding {
        return DokumentMelding(
            innhold = dokumentInnhold,
            mottaker = mottaker,
            configuration = dokumentConfiguration
        )
    }

    fun arbitraryDokumentConfiguration(
        sprak: Sprak = Sprak.Bokmal,
        iktLosningId: IktLosningId = arbitrayIktLosningId(),
        altinnConfiguration: AltinnConfiguration = arbitraryAltinnConfiguration(),
        offentligPostConfiguration: OffentligPostConfiguration = arbitraryOffentligPostConfiguration()
    ): DokumentConfiguration {
        return DokumentConfiguration(
            sprak = sprak,
            iktLosningId = iktLosningId,
            altinnConfiguration = altinnConfiguration,
            offentligPostConfiguration = offentligPostConfiguration
        )
    }

    fun arbitrayIktLosningId() = IktLosningId("OI-7254E00F5D2E4F93BADC4F4CC9FE25D4")

    fun arbitraryOffentligPostConfiguration(
        sikkerhetsNiva: Sikkerhetsniva = Sikkerhetsniva.Hoyeste,
        virkningsdato: LocalDate? = null,
        returhandtering: Returhandtering = Returhandtering.Makulere,
        iktlosningReferanse: IktLosningReferanse? = null,
        avsender: Postadresse = PostadresseNorsk(
            navn = "Vegvesenet",
            adresseLinjer = AdresseLinjer("Vegveien 1"),
            adresseType = AdresseType.Avsender,
            postNummer = "2065",
            postSted = "Lillehammer"
        )
    ): OffentligPostConfiguration {
        return OffentligPostConfiguration(
            sikkerhetsniva = sikkerhetsNiva,
            virkningsdato = virkningsdato,
            returhandtering = returhandtering,
            iktlosningReferanse = iktlosningReferanse,
            avsender = avsender
        )

    }

    fun arbitraryAltinnConfiguration(
        tjenestedefinisjon: AltinnTjenestedefinisjon = arbitraryAltinnTjenestedefinisjon(),
        tillatVideresending: Boolean = true,
        beskrivelse: String = "En beskrivelse til Altinn"
    ): AltinnConfiguration {
        return AltinnConfiguration(
            tjenestedefinisjon = tjenestedefinisjon,
            tillatVideresending = tillatVideresending,
            beskrivelse = beskrivelse
        )
    }

    fun arbitraryAltinnTjenestedefinisjon(): AltinnTjenestedefinisjon = AltinnTjenestedefinisjon(
        tjenestekode = Tjenestekode("2762"),
        tjenesteutgave = Tjenesteutgave("201109")
    )

    fun arbitraryPersonMottaker(
        personidentifikator: String = "25915997174",
        overstyrtAdresse: OverstyrtAdresse? = null
    ): Person {
        return Person(
            personIdentifikator = Fodselsnummer(personidentifikator),
            overstyrtAdresse = overstyrtAdresse
        )
    }

    fun arbitraryDokumentInnholdWithBevis(
        tittel: String = "Bevis",
        hoveddokument: Dokument = arbitraryDokument(),
        vedlegg: List<Dokument> = listOf(arbitraryDokument())
    ): DokumentInnhold {
        return DokumentInnhold(
            tittel = tittel,
            hoveddokument = hoveddokument,
            vedlegg = vedlegg,
            bevis = arbitraryBevis()
        )
    }

    fun arbitraryDokumentInnhold(
        tittel: String = "Vanlig melding",
        hoveddokument: Dokument = arbitraryDokument(),
        vedlegg: List<Dokument> = listOf(arbitraryDokument())
    ): DokumentInnhold {
        return DokumentInnhold(
            tittel = tittel,
            hoveddokument = hoveddokument,
            vedlegg = vedlegg,
            bevis = null
        )
    }

    fun arbitraryDokument(
        tittel: String = "En sensitiv tittel",
        bytes: ByteArray = readPdfBytesFromResources("statens_vegvesen.pdf")
    ): Dokument {
        return Dokument(
            tittel = tittel,
            bytes = bytes
        )
    }

    private fun readPdfBytesFromResources(filename: String): ByteArray {
        val classLoader = this.javaClass.classLoader
        val inputStream = classLoader.getResourceAsStream(filename)
            ?: throw IllegalArgumentException("file not found! $filename")
        return inputStream.readBytes()
    }

    fun arbitraryBevis(
        utstederVisningsnavn: String = "Vegvesenet",
        bakgrunnsfarge: ColorHex = ColorHex("#ababab"),
        utstedtTidspunkt: Instant = Instant.now(),
        gyldighetsperiode: Gyldighetsperiode? = null,
        bevisBevisHolder: BevisHolder = arbirtryBevisHolder(),
        tittel: String = "Påkrevd tittel",
        attributtList: List<BevisAttributt> = emptyList(),
        infoList: List<BevisInfo> = emptyList()
    ): Bevis {
        return Bevis(
            utstederVisningsnavn = utstederVisningsnavn,
            bakgrunnsfarge = bakgrunnsfarge,
            utstedtTidspunkt = utstedtTidspunkt,
            gyldighetsperiode = gyldighetsperiode,
            bevisHolder = bevisBevisHolder,
            bevisTittel = tittel,
            attributtList = attributtList,
            infoList = infoList,
            sprak = Sprak.Bokmal
        )
    }

    fun arbirtryBevisHolder(
        name: Name = Name("Ola", "Nordmann"),
        fodselsnummer: Fodselsnummer = Fodselsnummer("12345678901"),
        bevisAdresse: BevisAdresse? = null
    ): BevisHolder {
        return BevisHolder(
            name = name,
            fodselsnummer = fodselsnummer,
            bevisAdresse = bevisAdresse
        )
    }

    fun arbitraryOverstyrtAdresseNorsk(
        adresseLinjer: AdresseLinjer = AdresseLinjer("Vegveien 1"),
        postNummer: String = "1234",
        postSted: String = "Oslo",
    ): OverstyrtAdresse {
        return OvertstyrtAdresseNorsk(
            postnummer = postNummer,
            poststed = postSted,
            adresseLinjer = adresseLinjer
        )
    }

    fun arbitraryOverstyrtAdresseUtland(
        adresseLinjer: AdresseLinjer = AdresseLinjer("Roadstreet 1"),
        land: String = "SWEDEN",
    ): OverstyrtAdresse {
        return OvertstyrtAdresseUtenlandsk(
            adresseLinjer = adresseLinjer,
            land = land,
        )
    }


    fun arbitraryMeldingUuid(uuid: UUID = UUID.fromString("f47ac10b-58cc-4372-a567-0e02b2c3d479")): MeldingUuid = MeldingUuid(uuid)
    fun randomIdempotencyKey(): IdempotencyKey = IdempotencyKey(UUID.randomUUID().toString())
}
