import no.vegvesen.meldingsformidler.MeldingsformidlerEnvironment
import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Nested
import org.junit.jupiter.api.Test
import java.net.URI


class MeldingsformidlerEnvironmentTest {

    @Nested
    @DisplayName("Send melding environment uris set correctly")
    inner class SendMeldingEnvironment {
        @Test
        fun `create environment from localhost property`() {
            val expected = "localhost"

            val actual = MeldingsformidlerEnvironment.Localhost.baseUri.host

            assertThat(actual).contains(expected)
        }

        @Test
        fun `create environment from systemtest property`() {
            val expected = "mottak.utv.atlas.vegvesen.no"

            val actual = MeldingsformidlerEnvironment.Systemtest.baseUri.host

            assertThat(actual).contains(expected)
        }

        @Test
        fun `create environment from akseptansetest property`() {
            val expected = "mottak.test.atlas.vegvesen.no"

            val actual = MeldingsformidlerEnvironment.Akseptansetest.baseUri.host

            assertThat(actual).contains(expected)
        }

        @Test
        fun `create environment from produksjon property`() {
            val expected = "mottak.atlas.vegvesen.no"

            val actual = MeldingsformidlerEnvironment.Produksjon.baseUri.host

            assertThat(actual).contains(expected)
        }
    }


    @Test
    fun `can override environment with custom urls`() {
        val customEnvironment = MeldingsformidlerEnvironment.customOverride(
            sendMeldingEnvironmentBaseUri = URI.create("http://myrandomurl-path1.example.com"),
        )

        assertThat(customEnvironment.baseUri.host).contains("myrandomurl-path1.example.com")
    }
}
