package no.vegvesen.meldingsformidler

import ArbitraryTestData.arbitraryDokumentInnhold
import no.vegvesen.meldingsformidler.common.domain.Dokument
import no.vegvesen.meldingsformidler.send.DokumentInnhold
import no.vegvesen.meldingsformidler.send.DokumentInnhold.Companion.maxBytesTotalInAllDocuments
import no.vegvesen.meldingsformidler.send.EpostInnhold
import org.assertj.core.api.Assertions.assertThat
import org.junit.Assert.assertThrows
import org.junit.jupiter.api.Test

class InnholdTests {

    @Test
    fun `to long titel in dokument gets error`() {
        val longTitel = "*".repeat(101)
        assertThrows(IllegalArgumentException::class.java) {
            arbitraryDokumentInnhold(tittel = longTitel)
        }
    }

    @Test
    fun `to to many vedlegg gets error`() {
        assertThrows(IllegalArgumentException::class.java) {
            arbitraryDokumentInnhold(vedlegg = List(201) { Dokument("tittel", byteArrayOf(1, 2, 3, 4)) })
        }
    }

    @Test
    fun `total size of documents exceeds limit gets error`() {
        val halfOfMaxTotalByteSizeAndOneExtraBit = ByteArray(maxBytesTotalInAllDocuments + 1 / 2) { 0 }
        val exception = assertThrows(IllegalArgumentException::class.java) {
            DokumentInnhold(
                tittel = "Valid Title",
                hoveddokument = Dokument("Hoveddokument", halfOfMaxTotalByteSizeAndOneExtraBit),
                vedlegg = listOf(Dokument("Vedlegg", halfOfMaxTotalByteSizeAndOneExtraBit))
            )
        }
        assertThat(exception.message).contains("Total size of documents in bytes must be between")
    }

    @Test
    fun `epost with to long text gets error`() {
        val longText = "*".repeat(60001)
        assertThrows(IllegalArgumentException::class.java) {
            EpostInnhold(emne = "Valid Title", innhold = longText)
        }
    }

    @Test
    fun `epost with to long titel gets error`() {
        val longTitel = "*".repeat(101)
        assertThrows(IllegalArgumentException::class.java) {
            EpostInnhold(emne = longTitel, innhold = "Valid Text")
        }
    }

    @Test
    fun `epost with text and titel is created`() {
        EpostInnhold(emne = "Valid Title", innhold = "Vaild Text")
    }

    @Test
    fun `dokument with to long titel gets error`() {
        val longTitel = "*".repeat(101)
        assertThrows(IllegalArgumentException::class.java) {
            Dokument(tittel = longTitel, bytes = byteArrayOf(1, 2, 3, 4, 5))
        }
    }
}
