package no.vegvesen.meldingsformidler

import no.vegvesen.meldingsformidler.common.domain.Epostadresse
import no.vegvesen.meldingsformidler.common.domain.Fodselsnummer
import no.vegvesen.meldingsformidler.send.Organisasjon
import no.vegvesen.meldingsformidler.send.Person
import org.assertj.core.api.Assertions.assertThat
import org.junit.Assert.assertThrows
import org.junit.jupiter.api.Test

class MottakerTests {


    @Test
    fun `to long mottakerId gets error`() {
        val longMottakerId = "123456789012"
        assertThrows(IllegalArgumentException::class.java) {
            Person(longMottakerId)
        }
    }

    @Test
    fun `to long epostAdresse gets error`() {
        val longEpostadresse = "a".repeat(101) + "@example.com"
        assertThrows(IllegalArgumentException::class.java) {
            Epostadresse(longEpostadresse)
        }
    }

    @Test
    fun `fodelsenr must be only numbers`() {
        val mottakerIdWhitLetter = "123W5P78901"
        val errorMessage = assertThrows(IllegalArgumentException::class.java) {
            Person(Fodselsnummer(mottakerIdWhitLetter))
        }
        assertThat(errorMessage.message).isEqualTo("Fødselsnummer must consist of only digits.")
    }

    @Test
    fun `orgnr must be only numbers`() {
        val mottakerIdWhitLetter = "123W5P789"
        val errorMessage = assertThrows(IllegalArgumentException::class.java) {
            Organisasjon(mottakerIdWhitLetter)
        }
        assertThat(errorMessage.message).isEqualTo("Organisasjonsnummer must consist of only digits.")
    }
}
