package no.vegvesen.meldingsformidler.integrationtests

import no.vegvesen.meldingsformidler.common.domain.AdresseLinjer
import no.vegvesen.meldingsformidler.common.domain.AdresseType
import no.vegvesen.meldingsformidler.common.domain.AltinnTjenestedefinisjon
import no.vegvesen.meldingsformidler.common.domain.ArligRepeterendePeriode
import no.vegvesen.meldingsformidler.common.domain.ColorHex
import no.vegvesen.meldingsformidler.common.domain.DigitalPostVarsel
import no.vegvesen.meldingsformidler.common.domain.Dokument
import no.vegvesen.meldingsformidler.common.domain.Fodselsnummer
import no.vegvesen.meldingsformidler.common.domain.Gyldighetsperiode
import no.vegvesen.meldingsformidler.common.domain.IktLosningId
import no.vegvesen.meldingsformidler.common.domain.IktLosningReferanse
import no.vegvesen.meldingsformidler.common.domain.Postadresse
import no.vegvesen.meldingsformidler.common.domain.PostadresseNorsk
import no.vegvesen.meldingsformidler.common.domain.Returhandtering
import no.vegvesen.meldingsformidler.common.domain.Sikkerhetsniva
import no.vegvesen.meldingsformidler.common.domain.Sprak
import no.vegvesen.meldingsformidler.common.domain.TimeOfMonth
import no.vegvesen.meldingsformidler.common.domain.Tjenestekode
import no.vegvesen.meldingsformidler.common.domain.Tjenesteutgave
import no.vegvesen.meldingsformidler.common.domain.ValidMonths
import no.vegvesen.meldingsformidler.send.AltinnConfiguration
import no.vegvesen.meldingsformidler.send.Bevis
import no.vegvesen.meldingsformidler.send.BevisAdresse
import no.vegvesen.meldingsformidler.send.BevisAttributt
import no.vegvesen.meldingsformidler.send.BevisHolder
import no.vegvesen.meldingsformidler.send.BevisInfo
import no.vegvesen.meldingsformidler.send.DokumentConfiguration
import no.vegvesen.meldingsformidler.send.DokumentInnhold
import no.vegvesen.meldingsformidler.send.DokumentMelding
import no.vegvesen.meldingsformidler.send.DokumentMottaker
import no.vegvesen.meldingsformidler.send.EpostConfiguration
import no.vegvesen.meldingsformidler.send.EpostInnhold
import no.vegvesen.meldingsformidler.send.EpostMelding
import no.vegvesen.meldingsformidler.send.EpostMottaker
import no.vegvesen.meldingsformidler.send.IdempotencyKey
import no.vegvesen.meldingsformidler.send.Melding
import no.vegvesen.meldingsformidler.send.Name
import no.vegvesen.meldingsformidler.send.OffentligPostConfiguration
import no.vegvesen.meldingsformidler.send.Organisasjon
import no.vegvesen.meldingsformidler.send.OverstyrtAdresse
import no.vegvesen.meldingsformidler.send.OvertstyrtAdresseNorsk
import no.vegvesen.meldingsformidler.send.OvertstyrtAdresseUtenlandsk
import no.vegvesen.meldingsformidler.send.Person
import java.time.Instant
import java.time.LocalDate
import java.time.LocalTime
import java.time.MonthDay
import java.util.UUID

object IntegrationTestData {
    val fodselsnummerWithDigipost = Fodselsnummer("25915997174") // FATTET SENATOR
    val fodselsnummerWithEBoks = Fodselsnummer("10927796339") // OPPRØMT KAFETERIA
    val organisasjonsnummer = "313110494" // GJENNOMSIKTIG GOD MINK ANS
    val fodselsnummerReservert = Fodselsnummer("29890749036") // LOGISK KØ
    val fodselsnummerWithAltinn = Fodselsnummer("27859499002") // HELLIG KONTORBYGG
    val fodselsnummerUverifisert = Fodselsnummer("02448146394") // V-nummer lagret i Kundereg
    val dnummer = Fodselsnummer("41814033560") // HANDLEKRAFTIG DEIG
    val fodselsnummerDodAltinn = Fodselsnummer("01814797763") // BAKVERK FORSTANDIG
    val meldingsformidlerIktLosningId = IktLosningId("OI-7254E00F5D2E4F93BADC4F4CC9FE25D4")

    fun personWithDigipost() = Person(fodselsnummerWithDigipost)
    fun personWithEboks() = Person(fodselsnummerWithEBoks)
    fun organisasjon() = Organisasjon(organisasjonsnummer)
    fun personReservert() = Person(fodselsnummerReservert)
    fun personWithAltinn() = Person(fodselsnummerWithAltinn)
    fun uverifisertPerson() = Person(fodselsnummerUverifisert)
    fun dnummerPerson() = Person(dnummer)
    fun reservertPersonWithNorskOverstyrtAdresse() = Person(fodselsnummerReservert, overstyrtAdresseNorsk())
    fun reservertPersonWithUtenlandskOverstyrtAdresse() = Person(fodselsnummerReservert, overstyrtAdresseUtland())
    fun personDod() = Person(fodselsnummerDodAltinn)

    fun createEpostTestMelding(
        innhold: EpostInnhold = EpostInnhold(
            emne = "Testmelding",
            innhold = "Dette er en testmelding"
        ),
        mottaker: EpostMottaker = EpostMottaker("example@vegvesen.no"),
        configuration: EpostConfiguration = EpostConfiguration(meldingsformidlerIktLosningId)
    ): EpostMelding {
        return EpostMelding(
            innhold = innhold,
            mottaker = mottaker,
            configuration = configuration
        )
    }

    fun createTestDokumentMelding(
        dokumentInnhold: DokumentInnhold = dokumentInnhold(),
        mottaker: DokumentMottaker,
        configuration: DokumentConfiguration = meldingConfiguration()
    ): DokumentMelding {
        return DokumentMelding(
            innhold = dokumentInnhold,
            mottaker = mottaker,
            configuration = configuration
        )
    }

    fun bevisMelding(
        dokumentInnhold: DokumentInnhold = dokumentInnholdWithBevis(),
        mottaker: DokumentMottaker,
        configuration: DokumentConfiguration = meldingConfiguration()
    ): Melding {
        return DokumentMelding(
            innhold = dokumentInnhold,
            mottaker = mottaker,
            configuration = configuration
        )
    }

    private fun dokumentInnholdWithBevis(
        tittel: String = "Bevis",
        hoveddokument: Dokument = dokument(),
        vedlegg: List<Dokument> = listOf(dokument(tittel = "Vedlegg 1")),
    ): DokumentInnhold {
        return DokumentInnhold(
            tittel = tittel,
            hoveddokument = hoveddokument,
            vedlegg = vedlegg,
            bevis = bevis()
        )
    }

    private fun bevis(
        utstederVisningsnavn: String = "Vegvesenet",
        bakgrunnsfarge: ColorHex = ColorHex("#ababab"),
        utstedtTidspunkt: Instant = Instant.now(),
        gyldighetsperiode: Gyldighetsperiode? = arligRepeterendePeriode(),
        bevisHolder: BevisHolder = bevisHolder(),
        tittel: String = "Påkrevd tittel",
        attributtList: List<BevisAttributt> = emptyList(),
        infoList: List<BevisInfo> = emptyList()
    ): Bevis {
        return Bevis(
            utstederVisningsnavn = utstederVisningsnavn,
            bakgrunnsfarge = bakgrunnsfarge,
            utstedtTidspunkt = utstedtTidspunkt,
            gyldighetsperiode = gyldighetsperiode,
            bevisHolder = bevisHolder,
            bevisTittel = tittel,
            attributtList = attributtList,
            infoList = infoList,
            sprak = Sprak.Bokmal
        )
    }

    private fun arligRepeterendePeriode(): ArligRepeterendePeriode = ArligRepeterendePeriode(
        validMonths = ValidMonths(
            start = TimeOfMonth(LocalTime.now(), MonthDay.of(1, 1)),
            endInclusive = TimeOfMonth(LocalTime.now(), MonthDay.of(12, 31)),
        ), validYears = null
    )

    fun bevisHolder(
        name: Name = Name("Ola", "Nordmann"),
        fodselsnummer: Fodselsnummer = Fodselsnummer("12345678901"),
        bevisAdresse: BevisAdresse? = null
    ): BevisHolder {
        return BevisHolder(
            name = name,
            fodselsnummer = fodselsnummer,
            bevisAdresse = bevisAdresse
        )
    }


    fun dokumentInnhold(
        tittel: String = "Vanlig melding",
        hoveddokument: Dokument = dokument(),
        vedlegg: List<Dokument> = listOf(dokument(tittel = "Vedlegg 1")),
    ): DokumentInnhold {
        return DokumentInnhold(
            tittel = tittel,
            hoveddokument = hoveddokument,
            vedlegg = vedlegg,
            bevis = null
        )
    }

    private fun dokument(
        tittel: String = "En sensitiv tittel",
        bytes: ByteArray = readPdfBytesFromResources("statens_vegvesen.pdf")
    ): Dokument {
        return Dokument(
            tittel = tittel,
            bytes = bytes
        )
    }

    private fun readPdfBytesFromResources(filename: String): ByteArray {
        val classLoader = this.javaClass.classLoader
        val inputStream = classLoader.getResourceAsStream(filename)
            ?: throw IllegalArgumentException("file not found! $filename")
        return inputStream.readBytes()
    }


    private fun meldingConfiguration(
        sprak: Sprak = Sprak.Bokmal,
        iktLosningId: IktLosningId = meldingsformidlerIktLosningId,
        altinnConfiguration: AltinnConfiguration = altinnConfiguration(),
        offentligPostConfiguration: OffentligPostConfiguration = offentligPostConfiguration()
    ): DokumentConfiguration {
        return DokumentConfiguration(
            sprak = sprak,
            iktLosningId = iktLosningId,
            altinnConfiguration = altinnConfiguration,
            offentligPostConfiguration = offentligPostConfiguration
        )
    }

    private fun altinnConfiguration(
        tjenestedefinisjon: AltinnTjenestedefinisjon = altinnTjenestedefinisjon(),
        tillatVideresending: Boolean = true,
        beskrivelse: String = "En beskrivelse til Altinn"
    ): AltinnConfiguration {
        return AltinnConfiguration(
            tjenestedefinisjon = tjenestedefinisjon,
            tillatVideresending = tillatVideresending,
            beskrivelse = beskrivelse
        )
    }

    private fun altinnTjenestedefinisjon(): AltinnTjenestedefinisjon = AltinnTjenestedefinisjon(
        tjenestekode = Tjenestekode("2762"),
        tjenesteutgave = Tjenesteutgave("201109")
    )

    private fun overstyrtAdresseNorsk(
        adresseLinjer: AdresseLinjer = AdresseLinjer("Vegveien 1"),
        postNummer: String = "1234",
        postSted: String = "Oslo",
    ): OverstyrtAdresse {
        return OvertstyrtAdresseNorsk(
            postnummer = postNummer,
            poststed = postSted,
            adresseLinjer = adresseLinjer
        )
    }

    private fun overstyrtAdresseUtland(
        adresseLinjer: AdresseLinjer = AdresseLinjer("Roadstreet 1"),
        land: String = "SWEDEN",
    ): OverstyrtAdresse {
        return OvertstyrtAdresseUtenlandsk(
            adresseLinjer = adresseLinjer,
            land = land,
        )
    }

    private fun offentligPostConfiguration(
        sikkerhetsNiva: Sikkerhetsniva = Sikkerhetsniva.Hoyeste,
        virkningsdato: LocalDate? = null,
        returhandtering: Returhandtering = Returhandtering.Makulere,
        iktlosningReferanse: IktLosningReferanse? = null,
        avsender: Postadresse = PostadresseNorsk(
            navn = "Vegvesenet",
            adresseLinjer = AdresseLinjer("Vegveien 1"), AdresseType.Avsender,
            postNummer = "2065",
            postSted = "Lillehammer"
        ),
        digitalPostVarsel: DigitalPostVarsel = DigitalPostVarsel.VarselFraPostkasseOgVegvesenet,
    ): OffentligPostConfiguration {
        return OffentligPostConfiguration(
            sikkerhetsniva = sikkerhetsNiva,
            virkningsdato = virkningsdato,
            returhandtering = returhandtering,
            iktlosningReferanse = iktlosningReferanse,
            avsender = avsender,
            digitalPostVarsel = digitalPostVarsel,
        )

    }

    fun randomIdempotencyKey(): IdempotencyKey = IdempotencyKey(UUID.randomUUID().toString())
}
