package no.vegvesen.meldingsformidler.integrationtests

import no.vegvesen.meldingsformidler.common.domain.Kanal
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.bevisMelding
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.createEpostTestMelding
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.createTestDokumentMelding
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.dnummerPerson
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.dokumentInnhold
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.organisasjon
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.personDod
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.personReservert
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.personWithAltinn
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.personWithDigipost
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.personWithEboks
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.reservertPersonWithUtenlandskOverstyrtAdresse
import no.vegvesen.meldingsformidler.integrationtests.IntegrationTestData.uverifisertPerson
import no.vegvesen.meldingsformidler.send.Melding
import no.vegvesen.meldingsformidler.send.Person

class IntegrationTestInput {
    companion object {
        @JvmStatic
        fun altinn(): List<TestMeldingInput> = listOf(
            TestMeldingInput(
                name = "Person med Altinn",
                melding = createTestDokumentMelding(mottaker = personWithAltinn(), dokumentInnhold = dokumentInnhold(tittel = "Person, Altinn")),
                expectedKanal = Kanal.Altinn
            ),
            TestMeldingInput(
                name = "Organisasjon med Altinn",
                melding = createTestDokumentMelding(mottaker = organisasjon(), dokumentInnhold = dokumentInnhold(tittel = "Organisasjon, Altinn")),
                expectedKanal = Kanal.Altinn
            ),
            TestMeldingInput(
                name = "Person med D-nummer (midlertidig identifikasjonsnummer)",
                melding = createTestDokumentMelding(mottaker = dnummerPerson(), dokumentInnhold = dokumentInnhold(tittel = "D-nummer, Altinn")),
                expectedKanal = Kanal.Altinn
            ),
            TestMeldingInput(
                name = "Person som er død",
                melding = createTestDokumentMelding(
                    mottaker = personDod(),
                    dokumentInnhold = dokumentInnhold(tittel = "Død person, Altinn")
                ),
                expectedKanal = Kanal.Altinn
            ),
        )

        @JvmStatic
        fun digitalPost(): List<TestMeldingInput> = listOf(
            TestMeldingInput(
                name = "Person med Digipost",
                melding = createTestDokumentMelding(mottaker = personWithDigipost(), dokumentInnhold = dokumentInnhold(tittel = "Person, Digipost")),
                expectedKanal = Kanal.DigitalPost
            ),
            TestMeldingInput(
                name = "Person med Eboks",
                melding = createTestDokumentMelding(mottaker = personWithEboks(), dokumentInnhold = dokumentInnhold(tittel = "Person, Eboks")),
                expectedKanal = Kanal.DigitalPost
            ),
            TestMeldingInput(
                name = "Bevis til person med Digipost",
                melding = bevisMelding(mottaker = personWithDigipost(), dokumentInnhold = dokumentInnhold(tittel = "Bevis, Digipost")),
                expectedKanal = Kanal.DigitalPost
            ),
        )

        @JvmStatic
        fun brev(): List<TestMeldingInput> = listOf(
            TestMeldingInput(
                name = "Reservert mot digital post",
                melding = createTestDokumentMelding(mottaker = personReservert(), dokumentInnhold = dokumentInnhold(tittel = "Reservert, brev")),
                expectedKanal = Kanal.Brev
            ),
            TestMeldingInput(
                name = "Uverifisert person (V-nummer)",
                melding = createTestDokumentMelding(mottaker = uverifisertPerson(), dokumentInnhold = dokumentInnhold(tittel = "Uverifisert, brev")),
                expectedKanal = Kanal.Brev
            ),
            TestMeldingInput(
                name = "Overstyrt norsk adresse til person reservert mot digital post",
                melding = createTestDokumentMelding(
                    mottaker = IntegrationTestData.reservertPersonWithNorskOverstyrtAdresse(),
                    dokumentInnhold = dokumentInnhold(tittel = "Overstyrt adresse, brev"),
                ),
                expectedKanal = Kanal.Brev
            ),
            TestMeldingInput(
                name = "Overstyrt utenlandsk adresse til person reservert mot digital post",
                melding = createTestDokumentMelding(
                    mottaker = reservertPersonWithUtenlandskOverstyrtAdresse(),
                    dokumentInnhold = dokumentInnhold(tittel = "Overstyrt adresse utland, brev"),
                ),
                expectedKanal = Kanal.Brev
            ),
        )

        @JvmStatic
        fun epost(): List<TestMeldingInput> = listOf(
            TestMeldingInput(
                name = "Epost til person",
                melding = createEpostTestMelding(),
                expectedKanal = Kanal.Epost
            ),
        )

        @JvmStatic
        fun findKanal(): List<TestKanalInput> = listOf(
            TestKanalInput(name = "digitalpost", person = personWithDigipost(), expected = Kanal.DigitalPost),
            TestKanalInput(name = "digitalpost", person = personWithEboks(), expected = Kanal.DigitalPost),
            TestKanalInput(name = "brev", person = personReservert(), expected = Kanal.Brev),
            TestKanalInput(name = "brev", person = uverifisertPerson(), expected = Kanal.Brev),
            TestKanalInput(name = "altinn", person = personWithAltinn(), expected = Kanal.Altinn),
        )

    }
}

class TestMeldingInput(private val name: String, val melding: Melding, val expectedKanal: Kanal) {
    override fun toString(): String = name
}

class TestKanalInput(private val name: String, val person: Person, val expected: Kanal) {
    override fun toString(): String = name
}
